/*
Copyright 2025.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package v1alpha1

import (
	corev1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// EDIT THIS FILE!  THIS IS SCAFFOLDING FOR YOU TO OWN!
// NOTE: json tags are required.  Any new fields you add must have json tags for the fields to be serialized.

type BrowserStatus struct {
	Url     string `json:"url,omitempty"`
	Running bool   `json:"running,omitempty"`
}

type PortMapping struct {
	Protocol   corev1.Protocol `json:"protocol,omitempty"`
	HostPort   int32           `json:"host_port,omitempty"`
	TargetPort int32           `json:"target_port,omitempty"`
}

type Port struct {
	Protocol corev1.Protocol `json:"protocol,omitempty"`
	Port     int32           `json:"port,omitempty"`
}

type ServerStatus struct {
	Domain    string        `json:"domain,omitempty"`
	Running   bool          `json:"running,omitempty"`
	HostPorts []PortMapping `json:"host_ports,omitempty"`
	Args      []string      `json:"args,omitempty"`
	Command   []string      `json:"command,omitempty"`
}

type ServerSpec struct {
	Image      string   `json:"image,omitempty"`
	WorkingDir string   `json:"working_dir,omitempty"`
	Command    []string `json:"command,omitempty"`
	Args       []string `json:"args,omitempty"`
	On         bool     `json:"on,omitempty"`
	Ports      []Port   `json:"ports,omitempty"`
}

type BrowserSpec struct {
	On bool `json:"on,omitempty"`
}

// ServerManagerSpec defines the desired state of ServerManager
type ServerManagerSpec struct {
	Storage string `json:"storage,omitempty"`
	// INSERT ADDITIONAL SPEC FIELDS - desired state of cluster
	// Important: Run "make" to regenerate code after modifying this file
	Server  ServerSpec  `json:"server,omitempty"`
	Browser BrowserSpec `json:"browser,omitempty"`
}

// ServerManagerStatus defines the observed state of ServerManager
type ServerManagerStatus struct {
	// INSERT ADDITIONAL STATUS FIELD - define observed state of cluster
	// Important: Run "make" to regenerate code after modifying this file
	Server  ServerStatus  `json:"server,omitempty"`
	Browser BrowserStatus `json:"browser,omitempty"`
}

// +kubebuilder:object:root=true
// +kubebuilder:subresource:status

// ServerManager is the Schema for the servermanagers API
type ServerManager struct {
	metav1.TypeMeta   `json:",inline"`
	metav1.ObjectMeta `json:"metadata,omitempty"`

	Spec   ServerManagerSpec   `json:"spec,omitempty"`
	Status ServerManagerStatus `json:"status,omitempty"`
}

// +kubebuilder:object:root=true

// ServerManagerList contains a list of ServerManager
type ServerManagerList struct {
	metav1.TypeMeta `json:",inline"`
	metav1.ListMeta `json:"metadata,omitempty"`
	Items           []ServerManager `json:"items"`
}

func init() {
	SchemeBuilder.Register(&ServerManager{}, &ServerManagerList{})
}
